#version 430 core

struct Material {
	vec3 color;
	float shininess;
};

struct DirectionalLight {
    vec3 color;
	vec3 position;
};

uniform DirectionalLight light;
uniform Material material;
uniform float brightness;
uniform float specular_intensity;
uniform vec3 eyePos;
uniform float shades;

uniform sampler2D texture_diffuse1;
uniform sampler2D shadowMap;

in vec2 oTex;
in vec3 oPos;
in vec3 oNormal;
in vec4 oShadowCoord;

out vec4 FragColor;

float calculateShadow(float bias) {
	vec3 scoord = oShadowCoord.xyz / oShadowCoord.w;
	scoord = scoord * 0.5 + 0.5;
	if(scoord.z > 1.0f) return 0.0f;

	float shadow = 0.0f;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = 0; x <= 2; x++) {
		for(int y = 0; y <= 2; y++) {
			float pcfDepth = texture(shadowMap, scoord.xy + vec2(x-1, y-1) * texelSize).x; 
			shadow += scoord.z - bias > pcfDepth ? 1.0 : 0.0;        
		}    
	}

	return shadow/9.0;
}

void main() {
	vec3 normal = normalize(oNormal);
	vec3 lightDir = normalize(light.position);
	vec3 viewDir = normalize(eyePos - oPos);
	vec3 reflectDir = reflect(-lightDir, normal);
	float bias = max(0.005f * (1.0 - dot(normal, lightDir)), 0.0005f); 

	vec3 ambient = brightness * light.color;
	vec3 diffuse = max(dot(normal, lightDir), 0.0f) * light.color;
	vec3 specular = specular_intensity * pow(max(dot(viewDir, reflectDir), 0.0f), material.shininess) * light.color;

	FragColor = vec4((ambient + (1.0f - calculateShadow(bias))*(diffuse + specular)) * texture(texture_diffuse1, oTex).rgb, 1.0f);
} 