#version 430 core

uniform sampler2D colorTex;
uniform sampler2D depthTex;
uniform vec3 outlineColor;
uniform float far;
uniform float near;

in vec2 oTex;
out vec4 FragColor;

mat3 kernel_sobel_x = mat3 ( 
    1.0f, 2.0, 1.0f, 
    0.0f, 0.0, 0.0f, 
   -1.0f,-2.0,-1.0f 
);

mat3 kernel_sobel_y = mat3 ( 
    1.0f, 0.0f, -1.0f, 
    2.0f, 0.0f, -2.0f, 
    1.0f, 0.0f, -1.0f 
);

float reversePMatrix(float z) { //fixes depth values beeing broken due to projection
	return (2.0 * near) / (far + near - z * (far - near));
}

void main() {
	vec3 color = texture(colorTex, oTex).rgb;

	float sobel_x_kerneled = 0.0f;
	float sobel_y_kerneled = 0.0f;
	for (int x = 0; x < 3; x++) { //apply both 3x3 kernels
		for (int y = 0; y < 3; y++) {
			float depth = reversePMatrix(texelFetch(depthTex, ivec2(gl_FragCoord) + ivec2(x-1, y-1), 0).x);
            sobel_x_kerneled += depth * kernel_sobel_x[x][y];
			sobel_y_kerneled += depth * kernel_sobel_y[x][y];
		}
	}

	float sobel = length(vec2(sobel_x_kerneled, sobel_y_kerneled)); //euclidean distance of two soble filters is their combination 

	FragColor = vec4(color - vec3(sobel) * (vec3(1)-outlineColor), 1.0f);
	//FragColor = vec4(vec3(sobel), 1.0f); //outlines only
	//FragColor = vec4(vec3(reversePMatrix(texture(depthTex, oTex).x)), 1.0f); //depth buffer only
}